/* obj/db-config.h.  Generated automatically by configure.  */
/* obj/db-config.h.in.  Generated automatically from configure.in by autoheader.  */

/* Define to empty if the keyword does not work.  */
/* #undef const */

/* Define to `unsigned' if <sys/types.h> doesn't define.  */
/* #undef size_t */

/* Define if you have the ANSI C header files.  */
#define STDC_HEADERS 1

/* Define if your processor stores words with the most significant
   byte first (like Motorola and SPARC, unlike Intel and VAX).  */
#define WORDS_BIGENDIAN 1

/* #undef ssize_t */

/* #undef u_char */
/* #undef u_short */
/* #undef u_int */
/* #undef u_long */

/* #undef int8_t */
/* #undef u_int8_t */
/* #undef int16_t */
/* #undef u_int16_t */
/* #undef int32_t */
/* #undef u_int32_t */

/* #undef ssize_t */

/* #undef u_char */
/* #undef u_short */
/* #undef u_int */
/* #undef u_long */

/* #undef int8_t */
/* #undef u_int8_t */
/* #undef int16_t */
/* #undef u_int16_t */
/* #undef int32_t */
/* #undef u_int32_t */

/* The number of bytes in a int.  */
#define SIZEOF_INT 4

/* Define if you have the memmove function.  */
#define HAVE_MEMMOVE 1

/* Define if you have the mkstemp function.  */
#define HAVE_MKSTEMP 1

/* Define if you have the strerror function.  */
#define HAVE_STRERROR 1

/* Define if you have the <unistd.h> header file.  */
#define HAVE_UNISTD_H 1
